<?php
/**
 * Class that implements a WP_Theme_JSON_Schema migration.
 *
 * @package gutenberg
 */

/**
 * Class that removes the custom prefix for some properties
 * that did not land in 5.8.
 */
class WP_Theme_JSON_Schema_V1_Remove_Custom_Prefixes implements WP_Theme_JSON_Schema {
	/**
	 * Maps old properties to their new location within the schema's settings.
	 * This will be applied at both the defaults and individual block levels.
	 */
	const RENAMED_PATHS = array(
		'border.customColor'               => 'border.color',
		'border.customStyle'               => 'border.style',
		'border.customWidth'               => 'border.width',
		'typography.customFontStyle'       => 'typography.fontStyle',
		'typography.customFontWeight'      => 'typography.fontWeight',
		'typography.customLetterSpacing'   => 'typography.letterSpacing',
		'typography.customTextDecorations' => 'typography.textDecoration',
		'typography.customTextTransforms'  => 'typography.textTransform',
	);

	/**
	 * Removes the custom prefixes for a few properties that only worked in the plugin:
	 *
	 * 'border.customColor'               => 'border.color',
	 * 'border.customStyle'               => 'border.style',
	 * 'border.customWidth'               => 'border.width',
	 * 'typography.customFontStyle'       => 'typography.fontStyle',
	 * 'typography.customFontWeight'      => 'typography.fontWeight',
	 * 'typography.customLetterSpacing'   => 'typography.letterSpacing',
	 * 'typography.customTextDecorations' => 'typography.textDecoration',
	 * 'typography.customTextTransforms'  => 'typography.textTransform',
	 *
	 * @param array $old Data to migrate.
	 *
	 * @return array Data without the custom prefixes.
	 */
	public static function migrate( $old ) {
		// Copy everything.
		$new = $old;

		// Overwrite the things that change.
		if ( isset( $old['settings'] ) ) {
			$new['settings'] = self::process_settings( $old['settings'] );
		}

		return $new;
	}

	/**
	 * Processes the settings subtree.
	 *
	 * @param array $settings Array to process.
	 *
	 * @return array The settings in the new format.
	 */
	private static function process_settings( $settings ) {
		$new_settings = $settings;

		// Process any renamed/moved paths within default settings.
		self::rename_settings( $new_settings );

		// Process individual block settings.
		if ( isset( $new_settings['blocks'] ) && is_array( $new_settings['blocks'] ) ) {
			foreach ( $new_settings['blocks'] as &$block_settings ) {
				self::rename_settings( $block_settings );
			}
		}

		return $new_settings;
	}

	/**
	 * Processes a settings array, renaming or moving properties according to
	 * `self::RENAMED_PATHS`.
	 *
	 * @param array $settings Reference to settings either defaults or an individual block's.
	 * @return void
	 */
	private static function rename_settings( &$settings ) {
		foreach ( self::RENAMED_PATHS as $original => $renamed ) {
			$original_path = explode( '.', $original );
			$renamed_path  = explode( '.', $renamed );
			$current_value = _wp_array_get( $settings, $original_path, null );

			if ( null !== $current_value ) {
				gutenberg_experimental_set( $settings, $renamed_path, $current_value );
				self::unset_setting_by_path( $settings, $original_path );
			}
		}
	}

	/**
	 * Removes a property from within the provided settings by its path.
	 *
	 * @param array $settings Reference to the current settings array.
	 * @param array $path Path to the property to be removed.
	 *
	 * @return void
	 */
	private static function unset_setting_by_path( &$settings, $path ) {
		$tmp_settings = &$settings; // phpcs:ignore VariableAnalysis.CodeAnalysis.VariableAnalysis.UnusedVariable
		$last_key     = array_pop( $path );
		foreach ( $path as $key ) {
			$tmp_settings = &$tmp_settings[ $key ];
		}

		unset( $tmp_settings[ $last_key ] );
	}
}
